Subroutine rttov_transmit_ad( &
     & nfrequencies,    &! in
     & nchannels,       &! in
     & nprofiles,       &! in
     & nlevels,         &! in
     & channels,        &! in
     & polarisations,   &! in
     & lprofiles,       &! in
     & predictors,      &! in
     & predictors_ad,   &! inout
     & aux,             &! in
     & aux_ad,          &! inout
     & coef,            &! in
     & od_layer,        &! in
     & opdp_ref,        &! in
     & transmission,    &! in
     & transmission_ad ) ! inout

  ! Description:
  ! Adjoint of rttov_transmit_tl
  !  To calculate optical depths for a number of channels
  !  and profiles from every pressure level to space.
  ! To interpolate optical depths on to levels of radiative transfer model
  ! (which, at present, entails only surface transmittance, as
  ! other optical depths are on *rt* levels) and to convert
  ! optical depths to transmittances.
  !
  ! Code based on OPDEPAD and RTTAUAD from previous versions of RTTOV
  ! Only one profile per call
  !
  ! Adjoint variables
  ! input transmission_ad % tau_surf and transmission_ad % tau_layer
  ! set inside integrate_ad
  !
  ! input/output aux_ad
  !
  ! output predictors_ad initialised inside rttov_ad (need input
  !    intent for memory allocation in calling routine)
  !
  !
  ! Copyright:
  !    This software was developed within the context of
  !    the EUMETSAT Satellite Application Facility on
  !    Numerical Weather Prediction (NWP SAF), under the
  !    Cooperation Agreement dated 25 November 1998, between
  !    EUMETSAT and the Met Office, UK, by one or more partners
  !    within the NWP SAF. The partners in the NWP SAF are
  !    the Met Office, ECMWF, KNMI and MeteoFrance.
  !
  !    Copyright 2002, EUMETSAT, All Rights Reserved.
  !
  ! Method:
  !
  ! Current Code Owner: SAF NWP
  !
  ! History:
  ! Version   Date     Comment
  ! -------   ----     -------
  !  1.0    01/12/2002  New F90 code with structures (P Brunel A Smith)
  !  1.1    29/01/2003  Add WV Continuum and CO2 capability (P Brunel)
  !  1.2    04/12/2003  Optimisation (J Hague and D Salmond ECMWF)
  !  1.3    26/09/2003  Modified to allow for multiple polarisations (S English)
  !
  ! Code Description:
  !   Language:           Fortran 90.
  !   Software Standards: "European Standards for Writing and
  !     Documenting Exchangeable Fortran 90 Code".
  !
  ! Declarations:
  ! Modules used:

  ! Imported Parameters:

  Use rttov_const, Only: &
       & mwcldtop       ,&
       & sensor_id_mw


  Use rttov_types, Only : &
       & rttov_coef     ,&
       & predictors_Type,&
       & transmission_Type  ,&
       & profile_aux

  Use parkind1, Only : jpim     ,jprb
  Implicit None

  !subroutine arguments:
  Integer(Kind=jpim),   Intent(in)    :: nfrequencies
  Integer(Kind=jpim),   Intent(in)    :: nchannels
  Integer(Kind=jpim),   Intent(in)    :: nprofiles
  Integer(Kind=jpim),   Intent(in)    :: nlevels
  Integer(Kind=jpim),   Intent(in)    :: channels(nfrequencies)
  Integer(Kind=jpim),   Intent(in)    :: polarisations(nchannels,3)
  Integer(Kind=jpim),   Intent(in)    :: lprofiles(nfrequencies)
  Type(predictors_Type), Intent(in)    :: predictors( nprofiles )
  Type(predictors_Type), Intent(inout) :: predictors_ad( nprofiles )
  Type(rttov_coef),      Intent(in)    :: coef
  Type(profile_aux),     Intent(in)    :: aux( nprofiles )
  Type(profile_aux),     Intent(inout) :: aux_ad( nprofiles )
  Real(Kind=jprb),       Intent(in)    :: od_layer(nlevels,nchannels)
  Real(Kind=jprb),       Intent(in)    :: opdp_ref(nlevels,nfrequencies)
  Type(transmission_Type),Intent(in)   :: transmission
  Type(transmission_Type),Intent(inout):: transmission_ad

  !local variables:
  Real(Kind=jprb) :: od_layer_ad(nlevels,nfrequencies)
  Real(Kind=jprb) :: opticaldepth_ad(nlevels,nfrequencies)
  Real(Kind=jprb) :: od_surf_ad(nfrequencies)
  Real(Kind=jprb) :: tau_layer_freq(nlevels,nfrequencies) ! sat to layer transmission at each frequency
  Real(Kind=jprb) :: tau_surf_freq_ad(nfrequencies) ! sat to surface transmission at each frequency
  Real(Kind=jprb) :: tau_layer_freq_ad(nlevels,nfrequencies) ! sat to layer transmission at each frequency
  Real(Kind=jprb) :: od_singlelayer_freq_ad(nlevels,nfrequencies)
  Real(Kind=jprb),  Pointer :: debye_prof(:,:)
  Real(Kind=jprb),  Pointer :: debye_prof_ad(:,:)
  Integer(Kind=jpim) :: lev,chan,j,freq,kpol
  Integer(Kind=jpim) :: prof

  ! cloud liquid water local variables
  Real(Kind=jprb) :: zf, zf_sq, z34_dif, z45_dif, z1_sq, z2_sq, z1_div, z2_div
  Real(Kind=jprb) :: z1_den, z2_den, zastar, z1_prod, z2_prod, z3_prod, z4_prod
  Real(Kind=jprb) :: zbstar, zbstar_sq, za2star, za2star_sq, zdiv, zgstar
  Real(Kind=jprb) :: z1f_sq_z1_sq, z2f_sq_z2_sq

  Real(Kind=jprb) :: z34_dif_ad, z45_dif_ad, z1_sq_ad, z2_sq_ad, z1_div_ad, z2_div_ad
  Real(Kind=jprb) :: z1_den_ad, z2_den_ad, zastar_ad, z1_prod_ad, z2_prod_ad, z3_prod_ad, z4_prod_ad
  Real(Kind=jprb) :: zbstar_ad, zbstar_sq_ad, za2star_ad, za2star_sq_ad, zdiv_ad, zgstar_ad
  Real(Kind=jprb) :: z1f_sq_z1_sq_ad, z2f_sq_z2_sq_ad
  Integer(Kind=jpim) :: II

  !- End of header --------------------------------------------------------

  od_layer_ad(:,:)     = 0._JPRB
  opticaldepth_ad(:,:) = 0._JPRB

  !-----------------------------------------------------
  !5. Store transmittances for other polarisations
  !-----------------------------------------------------

  tau_layer_freq_ad(:, :)       = 0.0_JPRB
  od_singlelayer_freq_ad(:, :)  = 0.0_JPRB
  tau_surf_freq_ad(:)           = 0.0_JPRB

  Do j = 1, nchannels
     freq       = polarisations(j,2)               ! Frequency index
     kpol       = 1 + j - polarisations(freq,1)    ! Polarisation index
     prof       = lprofiles(freq)       ! Profile index
     tau_layer_freq(:,freq) = transmission % tau_layer(:,j)
     tau_layer_freq_ad(:,freq) = tau_layer_freq_ad(:,freq) + transmission_ad % tau_layer(:,j)
     od_singlelayer_freq_ad(:,freq) = od_singlelayer_freq_ad(:,freq) + &
                                                        & transmission_ad % od_singlelayer(:,j)
     tau_surf_freq_ad(freq) = tau_surf_freq_ad(freq) + transmission_ad % tau_surf(j)
     od_surf_ad(freq) =  transmission % tau_surf(j) * tau_surf_freq_ad(freq)
  End Do

  !-----------------------------------------------------
  !4. Compute optical depth and transmittance at surface
  !-----------------------------------------------------

  Do j = 1, nfrequencies
     prof       = lprofiles(j)
     chan       = polarisations(j,1)

     od_layer_ad(aux(prof) % nearestlev_surf,j) = od_layer_ad(aux(prof) % nearestlev_surf,j) +&
           & od_surf_ad(j) * (1._JPRB - aux(prof) % pfraction_surf )
     od_layer_ad(aux(prof) % nearestlev_surf-1,j) = od_layer_ad(aux(prof) % nearestlev_surf-1,j) +&
           & od_surf_ad(j) * aux(prof) % pfraction_surf
     aux_ad(prof) % pfraction_surf = aux_ad(prof) % pfraction_surf + od_surf_ad(j) *&
           & (  od_layer(aux(prof) % nearestlev_surf-1,chan) -    &
              & od_layer(aux(prof) % nearestlev_surf  ,chan) )
     od_surf_ad(j) = 0._JPRB
  End Do

  !-------------------------------------------
  !3. Convert optical depths to transmittances
  !-------------------------------------------
  od_layer_ad(:,:) = od_layer_ad(:,:) + tau_layer_freq_ad(:,:) * tau_layer_freq(:,:)

  !transmission_ad % tau_layer(:,:) = 0.

  !----------------------------------------
  !2. Compute layer to space optical depths
  !----------------------------------------
  ! notes: apply gamma correction; check value is sensible and constrain
  ! if necessary.

  Do lev = nlevels, 2, -1
     od_layer_ad(lev-1,:)   = od_layer_ad(lev-1,:)    + od_layer_ad(lev,:)
     opticaldepth_ad(lev,:) = opticaldepth_ad(lev,:)  + od_layer_ad(lev,:)
     od_layer_ad(lev,:)     = 0._JPRB
  End Do

  opticaldepth_ad(1,:) = opticaldepth_ad(1,:) + od_layer_ad(1,:)
  opticaldepth_ad(:,:) = opticaldepth_ad(:,:) - od_singlelayer_freq_ad(:,:)

  Do j = 1, nfrequencies
     chan = channels(j)
     opticaldepth_ad(:,j) = coef%ff_gam(chan) * opticaldepth_ad(:,j)
  End Do


  ! note that optical depth in the calculations is negative
  Where( opdp_ref(:,:) > 0.0_JPRB )
     opticaldepth_ad = 0.0_JPRB
  End Where

  !--------------------
  !1.6 add liquid water (MW only)
  !--------------------
  If ( coef % id_sensor == sensor_id_mw ) Then
     Do j = 1, nfrequencies
        chan = channels(j)
        prof = lprofiles(j)
        debye_prof    => aux(prof) % debye_prof
        debye_prof_ad => aux_ad(prof) % debye_prof
        If( predictors(prof) % ncloud >= 1 ) Then
           Do lev = mwcldtop, nlevels

              ! Repeat direct code
              zf           = coef % frequency_ghz(chan)
              zf_sq        = zf*zf
              z1_sq        = debye_prof(1,lev) * debye_prof(1,lev)
              z2_sq        = debye_prof(2,lev) * debye_prof(2,lev)
              z34_dif      = debye_prof(3,lev) - debye_prof(4,lev)
              z45_dif      = debye_prof(4,lev) - debye_prof(5,lev)
              z1f_sq_z1_sq = zf_sq + z1_sq
              z2f_sq_z2_sq = zf_sq + z2_sq
              z1_div       = 1.0_JPRB / z1f_sq_z1_sq
              z2_div       = 1.0_JPRB / z2f_sq_z2_sq
              z1_den       = z34_dif * z1_div
              z2_den       = z45_dif * z2_div
              zastar       = debye_prof(3,lev) - zf_sq * (z1_den + z2_den)
              z1_prod      = z34_dif * debye_prof(1,lev)
              z2_prod      = z1_prod * z1_div
              z3_prod      = z45_dif * debye_prof(2,lev)
              z4_prod      = z3_prod * z2_div
              zbstar       = -zf * (z2_prod + z4_prod)
              zbstar_sq    = zbstar * zbstar
              za2star      = zastar + 2.0_JPRB
              za2star_sq   = za2star * za2star
              zdiv         = za2star_sq + zbstar_sq
              zgstar       = -3.0_JPRB * zbstar / zdiv


              ! Now compute Adjoint code
              !opticaldepth_ad(lev,j)= opticaldepth_ad(lev,j)
              zgstar_ad = opticaldepth_ad(lev,j) *&
                    & (-1.5_JPRB * zf * predictors ( prof ) % clw(lev))
              predictors_ad ( prof ) % clw(lev) = predictors_ad ( prof ) % clw(lev) +&
                    & opticaldepth_ad(lev,j) * (-1.5_JPRB * zf * zgstar)

              zbstar_ad = -3.0_JPRB * zgstar_ad / zdiv
              zdiv_ad   =  3.0_JPRB * zgstar_ad * zbstar / (zdiv*zdiv)
              !zgstar_ad =  0.

              za2star_sq_ad = zdiv_ad
              zbstar_sq_ad  = zdiv_ad
              !zdiv_ad       = 0.

              za2star_ad    = 2.0_JPRB * za2star * za2star_sq_ad
              !za2star_sq_ad = 0.

              zastar_ad     = za2star_ad
              !za2star_ad    = 0.

              zbstar_ad     = zbstar_ad + 2.0_JPRB * zbstar * zbstar_sq_ad
              !zbstar_sq_ad  = 0.

              z2_prod_ad    = -zf * zbstar_ad
              z4_prod_ad    = -zf * zbstar_ad
              !zbstar_ad     = 0.

              z3_prod_ad    = z2_div  * z4_prod_ad
              z2_div_ad     = z3_prod * z4_prod_ad
              !z4_prod_ad    = 0.

              z45_dif_ad           = debye_prof(2,lev)             * z3_prod_ad
              debye_prof_ad(2,lev) = debye_prof_ad(2,lev) + z45_dif* z3_prod_ad
              !z3_prod_ad           = 0.

              z1_prod_ad    = z1_div  * z2_prod_ad
              z1_div_ad     = z1_prod * z2_prod_ad
              !z2_prod_ad    = 0.

              z34_dif_ad           = debye_prof(1,lev)              * z1_prod_ad
              debye_prof_ad(1,lev) = debye_prof_ad(1,lev) + z34_dif * z1_prod_ad
              !z1_prod_ad           = 0.

              debye_prof_ad(3,lev) = debye_prof_ad(3,lev) + zastar_ad
              z1_den_ad            =              -zf_sq  * zastar_ad
              z2_den_ad            =              -zf_sq  * zastar_ad
              !zastar_ad            = 0.

              z2_div_ad     = z2_div_ad  + z45_dif * z2_den_ad
              z45_dif_ad    = z45_dif_ad + z2_div  * z2_den_ad
              !z2_den_ad     = 0.

              z1_div_ad     = z1_div_ad  + z34_dif * z1_den_ad
              z34_dif_ad    = z34_dif_ad + z1_div  * z1_den_ad
              !z1_den_ad     = 0.

              z2f_sq_z2_sq_ad = -z2_div_ad / (z2f_sq_z2_sq * z2f_sq_z2_sq)
              !z2_div_ad       = 0.

              z1f_sq_z1_sq_ad = -z1_div_ad / (z1f_sq_z1_sq * z1f_sq_z1_sq)
              !z1_div_ad       = 0.

              z2_sq_ad        = z2f_sq_z2_sq_ad
              !z2f_sq_z2_sq_ad = 0.

              z1_sq_ad        = z1f_sq_z1_sq_ad
              !z1f_sq_z1_sq_ad = 0.

              debye_prof_ad(4,lev) = debye_prof_ad(4,lev) + z45_dif_ad
              debye_prof_ad(5,lev) = debye_prof_ad(5,lev) - z45_dif_ad
              !z45_dif_ad           = 0.

              debye_prof_ad(3,lev) = debye_prof_ad(3,lev) + z34_dif_ad
              debye_prof_ad(4,lev) = debye_prof_ad(4,lev) - z34_dif_ad
              !z34_dif_ad           = 0.

              debye_prof_ad(2,lev) = debye_prof_ad(2,lev) + z2_sq_ad *&
                    & 2.0_JPRB * debye_prof(2,lev)
              !z2_sq_ad             = 0.

              debye_prof_ad(1,lev) = debye_prof_ad(1,lev) + z1_sq_ad *&
                    & 2.0_JPRB * debye_prof(1,lev)
              !z1_sq_ad             = 0.

           End Do
        Endif
     End Do
  End If
  !-----------
  !1.5 add CO2
  !-----------

  If ( coef%nco2 > 0 ) Then
     Do j = 1, nfrequencies
        chan = channels(j)
        prof = lprofiles(j)
        Do lev=nlevels, 1, -1

           predictors_ad( prof ) % co2(:,lev) =&
                    & predictors_ad( prof ) % co2(:,lev) +&
                    & coef%co2(lev,chan,:) * opticaldepth_ad(lev,j)

        End Do
     End Do
  End If

  !------------------------------
  !1.4 add Water Vapour Continuum
  !------------------------------

  If ( coef%nwvcont > 0 ) Then
     Do j = 1, nfrequencies
        chan = channels(j)
        prof = lprofiles(j)
        Do lev=nlevels, 1, -1

           predictors_ad( prof ) % wvcont(:,lev) =&
                    & predictors_ad( prof ) % wvcont(:,lev) +&
                    & coef%wvcont(lev,chan,:) * opticaldepth_ad(lev,j)

        End Do
     End Do
  End If

  !-------------
  !1.3 add ozone
  !-------------

  If ( coef%nozone > 0 ) Then
     Do j = 1, nfrequencies
        chan = channels(j)
        prof = lprofiles(j)
        Do lev=nlevels, 1, -1
           Do ii=1,coef%nozone
             predictors_ad( prof ) % ozone(ii,lev) =&
                    & predictors_ad( prof ) % ozone(ii,lev) +&
                    & coef%ozone(lev,chan,ii) * opticaldepth_ad(lev,j)
           End Do
        End Do
     End Do
  End If

  !--------------------
  !1.2 add water vapour
  !--------------------

  Do j = 1, nfrequencies
     chan = channels(j)
     prof = lprofiles(j)
     Do lev=nlevels, 1, -1
       Do ii=1,coef%nwater
        predictors_ad( prof ) % watervapour(ii,lev) =&
                 & predictors_ad( prof ) % watervapour(ii,lev) +   &
                 & coef%watervapour(lev,chan,ii) * opticaldepth_ad(lev,j)
       End Do
     End Do
  End Do

  !--------------------------
  !1.1 start with mixed gases
  !--------------------------

  Do j = 1, nfrequencies
     chan = channels(j)
     prof = lprofiles(j)
     Do lev=nlevels, 1, -1
       Do ii=1,coef%nmixed
        predictors_ad( prof ) % mixedgas(ii,lev) =&
                 & predictors_ad( prof ) % mixedgas(ii,lev) +&
                 & coef%mixedgas(lev,chan,ii) * opticaldepth_ad(lev,j)
       End Do
     End Do
  End Do

End Subroutine rttov_transmit_ad
